﻿
#region Using Directives

using System;
using System.Windows.Input;

#endregion

namespace TuneDepot.ViewModels
{

    /// <summary>
    /// Represents a command, who's Execute and CanExecute Methods can be defined by delegates.
    /// </summary>
    public class RelayCommand : ICommand
    {

        #region Constructors

        /// <summary>
        /// Creates a new relay command and sets the Execute and CanExecute behaviour.
        /// </summary>
        /// <param name="execute">
        /// The Execute behaviour.
        /// </param>
        /// <param name="canExecute">
        /// The CanExecute behaviour.
        /// </param>
        public RelayCommand(Action<object> execute, Predicate<object> canExecute)
        {

            if (execute == null)
                throw new ArgumentNullException("execute");

            this.execute = execute;
            this.canExecute = canExecute;

        }

        /// <summary>
        /// Creates a new relay command and defines its Execute behaviour.
        /// </summary>
        /// <param name="execute">
        /// The execute behaviour.
        /// </param>
        public RelayCommand(Action<object> execute)
            : this(execute, null)
        {
        }

        #endregion

        #region Private Fields

        /// <summary>
        /// The delegate, which is executed, when the command is executed.
        /// </summary>
        readonly Action<object> execute;

        /// <summary>
        /// The delegate, which is executed to determine whether the command can be executed.
        /// </summary>
        readonly Predicate<object> canExecute;

        #endregion

        #region ICommand Members

        /// <summary>
        /// Return whether the command can be executed.
        /// </summary>
        /// <param name="parameter">
        /// The parameters.
        /// </param>
        /// <returns>
        /// Return true, when the command can be executed and false otherwise.
        /// </returns>
        public bool CanExecute(object parameter)
        {

            return canExecute == null ? true : canExecute(parameter);
        
        }

        /// <summary>
        /// Executes the command.
        /// </summary>
        /// <param name="parameter">
        /// The parameters.
        /// </param>
        public void Execute(object parameter)
        {

            execute(parameter);

        }

        /// <summary>
        /// The event, which is raised when the value, that determines whether the command can execute, has changed.
        /// </summary>
        public event EventHandler CanExecuteChanged;

        #endregion

    }

}
